//---------------------------------------------------------------------------------------
// This file is part of the Lomse library.
// Copyright (c) 2010-present, Lomse Developers
//
// Licensed under the MIT license.
//
// See LICENSE and NOTICE.md files in the root directory of this source tree.
//---------------------------------------------------------------------------------------

#ifndef __LOMSE_CONFIG_H__
#define __LOMSE_CONFIG_H__

//==================================================================
// Template configuration file.
// Variables are replaced by CMake settings
//==================================================================

//---------------------------------------------------------------------------------------
// Paths, for fonts and unit tests resources
//
//    LOMSE_FONTS_PATH
//        - For Linux this path is a fallback path in case Bravura.otf font is not 
//          found in systems fonts.
//        - For Windows this path is to look for the Bravura.otf font.
//        - For platforms other than Linux and Windows the absolute path to the fonts
//          directory to use must be specified here.
//      Nevertheless, at run time the application using Lomse can set this path by
//      invoking method LomseDoorway::set_default_fonts_path(const string& fontsPath)
//
//    TESTLIB_SCORES_PATH
//        Absolute path for tests scores used in unit tests.
//
//    TESTLIB_FONTS_PATH
//        Absolute path for fonts used in unit tests.
//
//---------------------------------------------------------------------------------------
#define LOMSE_FONTS_PATH            "/builddir/build/BUILD/lomse-0.30.0git20241022-build/lomse-2067a01cd652a723e22746683d271b43f8d2ff38/fonts/"
#define TESTLIB_SCORES_PATH         "/builddir/build/BUILD/lomse-0.30.0git20241022-build/lomse-2067a01cd652a723e22746683d271b43f8d2ff38/test-scores/"
#define TESTLIB_FONTS_PATH          "/builddir/build/BUILD/lomse-0.30.0git20241022-build/lomse-2067a01cd652a723e22746683d271b43f8d2ff38/fonts/"


//---------------------------------------------------------------------------------------
// platform and compiler
//---------------------------------------------------------------------------------------
#define LOMSE_PLATFORM_WIN32      0
#define LOMSE_PLATFORM_UNIX       1
#define LOMSE_PLATFORM_APPLE      0
#define LOMSE_COMPILER_MSVC       0


//---------------------------------------------------------------------------------------
// what are you doing?
//    - creating the library as shared library   LOMSE_CREATE_DLL == 1
//    - using the library as shared library      LOMSE_USE_DLL == 1
//    - creating the library as static library   LOMSE_CREATE_DLL == 0 
//    - using the library as static library      LOMSE_USE_DLL == 0
//---------------------------------------------------------------------------------------
#define LOMSE_CREATE_DLL    0
#define LOMSE_USE_DLL       0

//---------------------------------------------------------------------------------------
// build options
//---------------------------------------------------------------------------------------
#define ON 1
#define OFF 0

// Debug build: include debug options
#define LOMSE_DEBUG                 OFF 

// Accept without warning/error LDP v1.5 syntax
#define LOMSE_COMPATIBILITY_LDP_1_5     ON

// Enable debug logs. It is independent of build mode: debug or release
#define LOMSE_ENABLE_DEBUG_LOGS     OFF

// Enable compressed formats (requires zlib)
#define LOMSE_ENABLE_COMPRESSION    ON

// Enable png format (requires pnglib and zlib)
#define LOMSE_ENABLE_PNG    ON

// Enable threads (requires pthreads). If not enabled, ScorePlayer will not be included
#define LOMSE_ENABLE_THREADS    ON


#endif  // __LOMSE_CONFIG_H__

